@@ -6,7 +6,7 @@
  * Descr.: atom manipulation management - C part                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom_c.c,v 1.6 2001/12/07 21:05:09 diaz Exp $ */
+/* $Id: atom_c.c,v 1.11 2003/01/06 12:18:05 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -74,10 +74,10 @@ Prolog_Prototype(ATOM_CONCAT_ALT, 0);
 Prolog_Prototype(SUB_ATOM_ALT, 0);
 Prolog_Prototype(CURRENT_ATOM_ALT, 0);
 
-#define MALLOC_STR(n) \
-   if (n<0) \
-     return FALSE; \
-   str=(char *) Malloc(n+1)
+#define MALLOC_STR(n)				\
+   if (n<0)					\
+     return FALSE;				\
+   str = (char *) Malloc(n + 1)
 
 
 
@@ -187,10 +187,8 @@ Atom_Concat_3(WamWord atom1_word, WamWord atom2_word, WamWord atom3_word)
       Create_Choice_Point((CodePtr) Prolog_Predicate(ATOM_CONCAT_ALT, 0), 4);
     }
 
-  Get_Atom(atom_void, atom1_word);
-  Get_Atom_Tagged(atom3_word, atom2_word);
-
-  return TRUE;
+  return Get_Atom(atom_void, atom1_word) &&
+    Get_Atom_Tagged(atom3_word, atom2_word);
 }
 
 
@@ -235,14 +233,13 @@ Atom_Concat_Alt_0(void)
   MALLOC_STR(l);
   strncpy(str, name, l + 1);
   str[l] = '\0';
-  Get_Atom(Create_Malloc_Atom(str), atom1_word);
+  if (!Get_Atom(Create_Malloc_Atom(str), atom1_word))
+    return FALSE;
 
   l = patom3->prop.length - l;
   MALLOC_STR(l);
   strcpy(str, p);
-  Get_Atom(Create_Malloc_Atom(str), atom2_word);
-
-  return TRUE;
+  return Get_Atom(Create_Malloc_Atom(str), atom2_word);
 }
 
 
@@ -832,8 +829,8 @@ String_To_Number(char *str, WamWord number_word)
     }
 #endif
 
-  stm = Add_Str_Stream(TRUE, str);
-  pstm = stm_tbl + stm;
+  stm = Add_Str_Stream(str, TERM_STREAM_ATOM);
+  pstm = stm_tbl[stm];
 
   word = Read_Number(pstm);
   eof = (Stream_Peekc(pstm) == EOF);
