@@ -6,7 +6,7 @@
  * Descr.: byte-code support                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.c,v 1.6 2001/12/07 21:05:09 diaz Exp $ */
+/* $Id: bc_supp.c,v 1.9 2003/01/06 12:18:05 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -216,9 +216,8 @@ static WamCont BC_Emulate_Clause(DynCInf *clause);
 
 static WamCont BC_Emulate_Byte_Code(BCWord *bc);
 
-static
-  void Prep_Debug_Call(int func, int arity, int caller_func,
-		       int caller_arity);
+static void Prep_Debug_Call(int func, int arity, int caller_func,
+			    int caller_arity);
 
 
 
@@ -751,7 +750,8 @@ BC_Call_Terminal_Pred_3(WamWord pred_word, WamWord call_info_word,
       return debug_call_code;
     }
 
-  if ((pred = Lookup_Pred(func, arity)) == NULL)
+  pred = Lookup_Pred(func, arity);
+  if (pred == NULL)
     {				/* case: fail/0 from '$call_from_debugger' */
       if (func != atom_fail || arity != 0)
 	{
