@@ -6,7 +6,7 @@
  * Descr.: debugger - C part                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: debugger_c.c,v 1.6 2001/12/20 20:50:57 diaz Exp $ */
+/* $Id: debugger_c.c,v 1.9 2003/01/06 12:18:07 diaz Exp $ */
 
 #include <string.h>
 #include <stdlib.h>
@@ -340,8 +340,8 @@ Debug_Wam(void)
   char *prompt = "(wam debug) ";
   int ret;
 
-  pstm_i = stm_tbl + stm_debugger_input;
-  pstm_o = stm_tbl + stm_debugger_output;
+  pstm_i = stm_tbl[stm_debugger_input];
+  pstm_o = stm_tbl[stm_debugger_output];
 
   Stream_Printf(pstm_o, "Welcome to the WAM debugger - experts only\n");
 
