@@ -6,7 +6,7 @@
  * Descr.: parser support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: parse_supp.c,v 1.6 2001/12/20 20:50:57 diaz Exp $ */
+/* $Id: parse_supp.c,v 1.9 2003/01/06 12:18:08 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -159,13 +159,16 @@ Read_Next_Token(Bool comma_is_punct)
  *                                                                         *
  * Returns a Prolog term as a WAM word or NOT_A_WAM_WORD on syntax error.  *
  * parse_end_of_term controls the end of term (see parse_supp.h).          *
- * Uses the value of: FLAG_DOUBLE_QUOTES                                   *
+ * Uses the value of: FLAG_DOUBLE_QUOTES, FLAG_BACK_QUOTES                 *
  *-------------------------------------------------------------------------*/
 WamWord
 Read_Term(StmInf *pstm, int parse_end_of_term)
 {
   int jmp_val;
   WamWord term;
+  int save_use_le_prompt = use_le_prompt;
+
+  use_le_prompt = 1;
 
   parse_nb_var = 0;
   last_read_line = -1;
@@ -221,6 +224,7 @@ Read_Term(StmInf *pstm, int parse_end_of_term)
     }
 
 finish:
+  use_le_prompt = save_use_le_prompt;
   return term;
 }
 
