@@ -6,7 +6,7 @@
  * Descr.: pretty print clause management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pretty_c.c,v 1.6 2001/12/07 21:05:10 diaz Exp $ */
+/* $Id: pretty_c.c,v 1.9 2003/01/06 12:18:09 diaz Exp $ */
 
 #define OBJ_INIT Pretty_Initializer
 
@@ -140,7 +140,7 @@ Portray_Clause_2(WamWord sora_word, WamWord term_word)
 
   stm = (sora_word == NOT_A_WAM_WORD)
     ? stm_output : Get_Stream_Or_Alias(sora_word, STREAM_CHECK_OUTPUT);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   last_output_sora = sora_word;
   Check_Stream_Type(stm, TRUE, FALSE);
