@@ -6,7 +6,7 @@
  * Descr.: stream support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_supp.c,v 1.7 2001/12/20 20:50:57 diaz Exp $ */
+/* $Id: stream_supp.c,v 1.15 2003/01/06 12:18:10 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -93,13 +93,15 @@ extern void clearerr(FILE *stream);
 static void Init_Stream_Supp();
 void (*init_stream_supp)() = Init_Stream_Supp; /* overwrite var of engine.c */
 
-static int atom_stream;
+static int atom_constant_term_stream;
+
 static WamWord stream_1;
 
 static WamWord word_current_input_stream;
 static WamWord word_current_output_stream;
 
-static StrSInf glob_str_stream = { NULL, NULL, 0 };
+static StrSInf static_str_stream_rd = { NULL, NULL, 0 }; /* input */
+static StrSInf static_str_stream_wr = { NULL, NULL, 0 }; /* output */
 
 
 #ifndef NO_USE_LINEDIT
@@ -116,6 +118,11 @@ static char *tty_ptr = NULL;	/* current pointer into the buff  */
 
 static int Find_Free_Stream(void);
 
+static void Del_Aliases_Of_Stream(int stm);
+
+static void Update_Mirrors_To_Del_Stream(int stm);
+
+static Bool Remove_In_Stream_List(int stm, StmLst **p_start);
 
 
 #ifndef NO_USE_LINEDIT
@@ -130,6 +137,8 @@ static void TTY_Clearerr(void);
 
 static int Basic_Call_Fct_Getc(StmInf *pstm);
 
+static void Basic_Call_Fct_Putc(int c, StmInf *pstm);
+
 static int Str_Stream_Getc(StrSInf *str_stream);
 
 static void Str_Stream_Putc(int c, StrSInf *str_stream);
@@ -141,27 +150,30 @@ static void Str_Stream_Putc(int c, StrSInf *str_stream);
 /*-------------------------------------------------------------------------*
  * INIT_STREAM_SUPP                                                        *
  *                                                                         *
- * no declare as other initializers, since we must be sure it has been     *
+ * no declared as other initializers, since we must be sure it has been    *
  * initialized before others.                                              *
  *-------------------------------------------------------------------------*/
 static void
 Init_Stream_Supp(void)
 {
-  StmProp prop;
-  int istty;
-  StmFct fct_putc = NULL;
-  StmFct fct_flush = NULL;
+#ifndef NO_USE_LINEDIT
+  StmInf *pstm;
+#endif
+
+  stm_tbl_size = 32;
+  stm_tbl = (StmInf **) Calloc(stm_tbl_size, sizeof(StmInf *));
+  stm_last_used = -1;
 
   alias_tbl = Hash_Alloc_Table(START_ALIAS_TBL_SIZE, sizeof(AliasInf));
 
   atom_stream = Create_Atom("$stream");
   stream_1 = Functor_Arity(atom_stream, 1);
 
+  atom_constant_term_stream = Create_Atom("constant term stream");
+
   word_current_input_stream = Tag_ATM(Create_Atom("current_input_stream"));
   word_current_output_stream = Tag_ATM(Create_Atom("current_output_stream"));
 
-  atom_glob_stream_alias = Create_Atom("$glob_stream_alias");
-
   atom_user_input = Create_Atom("user_input");
   atom_user_output = Create_Atom("user_output");
 
@@ -199,59 +211,43 @@ Init_Stream_Supp(void)
   atom_eof = Create_Atom("eof");
 
   le_prompt = "";
+  use_le_prompt = TRUE;
 
-  istty = (use_gui) || isatty(0);
-
-  prop.mode = STREAM_MODE_READ;
-  prop.input = TRUE;
-  prop.output = FALSE;
-  prop.text = TRUE;
-  prop.reposition = FALSE;
-  prop.eof_action = STREAM_EOF_ACTION_RESET;
-  prop.buffering = (istty) ? STREAM_BUFFERING_LINE : STREAM_BUFFERING_BLOCK;
-  prop.special_close = FALSE;
-  prop.other = 0;
+  stm_stdin = Add_Stream_For_Stdio_Desc(stdin, atom_user_input,
+					STREAM_MODE_READ, TRUE);
 
 #ifndef NO_USE_LINEDIT
-  if (istty)
-    stm_stdin = Add_Stream(atom_user_input, (long) stdin, prop,
-			   (StmFct) TTY_Getc, STREAM_FCT_UNDEFINED,
-			   STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
-			   STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
-			   (StmFct) TTY_Clearerr);
-  else
+  if ((use_gui) || isatty(0))
+    {
+      pstm = stm_tbl[stm_stdin];
+      pstm->fct_getc = (StmFct) TTY_Getc;
+      pstm->fct_putc = STREAM_FCT_UNDEFINED;
+      pstm->fct_flush = STREAM_FCT_UNDEFINED;
+      pstm->fct_close = STREAM_FCT_UNDEFINED;
+      pstm->fct_tell = STREAM_FCT_UNDEFINED;
+      pstm->fct_seek = STREAM_FCT_UNDEFINED;
+      pstm->fct_clearerr = (StmFct) TTY_Clearerr;
+    }
 #endif
-    stm_stdin = Add_Stream(atom_user_input, (long) stdin, prop,
-			   NULL, NULL, NULL, STREAM_FCT_UNDEFINED, NULL,
-			   NULL, NULL);
-
   Add_Alias_To_Stream(atom_user_input, stm_stdin);
   stm_input = stm_stdin;
 
-  istty = (use_gui) || isatty(1);
 
-  prop.mode = STREAM_MODE_APPEND;
-  prop.input = FALSE;
-  prop.output = TRUE;
-  prop.text = TRUE;
-  prop.reposition = FALSE;
-  prop.eof_action = STREAM_EOF_ACTION_RESET;
-  prop.buffering = (istty) ? STREAM_BUFFERING_LINE : STREAM_BUFFERING_BLOCK;
-  prop.special_close = FALSE;
-  prop.other = 0;
+
+  stm_stdout = Add_Stream_For_Stdio_Desc(stdout, atom_user_output,
+					 STREAM_MODE_WRITE, TRUE);
 
 #if !defined(NO_USE_LINEDIT) && defined(M_ix86_win32)
 		/* ok for both GUI and console EOM<->ANSI conversion */
-  if (le_hook_put_char && istty)
-    fct_putc = (StmFct) le_hook_put_char;
+  pstm = stm_tbl[stm_stdout];
+  pstm->prop.buffering = STREAM_BUFFERING_LINE;
+  if (le_hook_put_char && isatty(1))
+    pstm->fct_putc = (StmFct) le_hook_put_char;
 
-  if (le_hook_flush && istty)
-    fct_flush = (StmFct) le_hook_flush;
+  if (le_hook_flush && isatty(1))
+    pstm->fct_flush = (StmFct) le_hook_flush;
 #endif
 
-  stm_stdout = Add_Stream(atom_user_output, (long) stdout, prop,
-			  NULL, fct_putc, fct_flush,
-			  STREAM_FCT_UNDEFINED, NULL, NULL, NULL);
   Add_Alias_To_Stream(atom_user_output, stm_stdout);
   stm_output = stm_stdout;
 
@@ -260,62 +256,112 @@ Init_Stream_Supp(void)
 
   Add_Alias_To_Stream(atom_top_level_input, stm_top_level_input);
   Add_Alias_To_Stream(atom_top_level_output, stm_top_level_output);
+
   Add_Alias_To_Stream(atom_debugger_input, stm_debugger_input);
   Add_Alias_To_Stream(atom_debugger_output, stm_debugger_output);
 }
 
 
 
+
 #ifndef FOR_EXTERNAL_USE
 
 /*-------------------------------------------------------------------------*
- * GET_STREAM_MODE                                                         *
+ * PROP_AND_STDIO_MODE                                                     *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 StmProp
-Get_Stream_Mode(WamWord mode_word, Bool only_rw, char *open_str)
+Prop_And_Stdio_Mode(int mode, Bool text, char *open_str)
 {
-  int atom;
   StmProp prop;
 
-  atom = Rd_Atom_Check(mode_word);
+  prop.mode = mode;
 
-  if (atom == atom_read)
+  switch(mode)
     {
-      prop.mode = STREAM_MODE_READ;
+    case STREAM_MODE_READ:
       prop.input = TRUE;
       prop.output = FALSE;
-      strcpy(open_str, "r");
-      goto end;
-    }
+      *open_str++ = 'r';
+      break;
 
-  if (atom == atom_write)
-    {
-      prop.mode = STREAM_MODE_WRITE;
+    case STREAM_MODE_WRITE:
       prop.input = FALSE;
       prop.output = TRUE;
-      strcpy(open_str, "w");
-      goto end;
-    }
+      *open_str++ = 'w';
+      break;
 
-  if (only_rw)
-    Pl_Err_Domain(domain_io_mode, mode_word);
-
-  if (atom == atom_append)
-    {
-      prop.mode = STREAM_MODE_APPEND;
+    case STREAM_MODE_APPEND:
       prop.input = FALSE;
       prop.output = TRUE;
-      strcpy(open_str, "a");
-      goto end;
+      *open_str++ = 'a';
     }
+      
+  prop.text = text;
+  prop.reposition = TRUE;
+  prop.eof_action = STREAM_EOF_ACTION_EOF_CODE;
+  prop.buffering = STREAM_BUFFERING_BLOCK;
+  prop.special_close = FALSE;
+  prop.other = 0;
 
-  Pl_Err_Domain(domain_io_mode, mode_word);
+  *open_str++ = (text) ? 't' : 'b';
+  *open_str = '\0';
 
-end:
   return prop;
 }
 
+
+
+
+/*-------------------------------------------------------------------------*
+ * ADD_STREAM_FOR_STDIO_DESC                                               *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Add_Stream_For_Stdio_Desc(FILE *f, int atom_path, int mode, int text)
+{
+  char open_str[10];
+  StmProp prop = Prop_And_Stdio_Mode(mode, text, open_str);
+
+  prop.reposition = Stdio_Is_Repositionable(f);
+  prop.buffering = (prop.reposition) ? STREAM_BUFFERING_BLOCK :
+    STREAM_BUFFERING_LINE;
+
+  Stdio_Set_Buffering(f, prop.buffering);
+  if (isatty(fileno(f)))
+    prop.eof_action = STREAM_EOF_ACTION_RESET;
+
+  return Add_Stream(atom_path, (long) f, prop,
+		    NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * ADD_STREAM_FOR_STDIO_FILE                                               *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Add_Stream_For_Stdio_File(char *path, int mode, Bool text)
+{
+  FILE *f;
+  char open_str[10];
+  int atom_path;
+  
+  Prop_And_Stdio_Mode(mode, text, open_str); /* only for open_str */
+
+  if ((f = fopen(path, open_str)) == NULL)
+    return -1;
+
+  atom_path = Create_Allocate_Atom(path);
+
+  return Add_Stream_For_Stdio_Desc(f, atom_path, mode, text);
+}
+
+
+
+
 #endif /* !FOR_EXTERNAL_USE */
 
 /*-------------------------------------------------------------------------*
@@ -332,6 +378,8 @@ Init_Stream_Struct(int atom_file_name, long file, StmProp prop,
   pstm->atom_file_name = atom_file_name;
   pstm->file = file;
   pstm->prop = prop;
+  pstm->mirror = NULL;
+  pstm->mirror_of = NULL;
 
 #define INIT_FCT(f, d) pstm->f = (f) ? f : (StmFct) d
 
@@ -343,8 +391,8 @@ Init_Stream_Struct(int atom_file_name, long file, StmProp prop,
   INIT_FCT(fct_seek, fseek);
   INIT_FCT(fct_clearerr, clearerr);
 
-  /* Works only because putc will be called with c as 1st and flush's arg
-     is ignored */
+  /* Works only because putc will be called with c as 1st arg
+     and flush's arg is ignored */
 
   pstm->eof_reached = FALSE;
   PB_Init(pstm->pb_char);
@@ -378,13 +426,11 @@ Add_Stream(int atom_file_name, long file, StmProp prop,
     Fatal_Error(ERR_TELL_OR_SEEK_UNDEFINED);
 
 
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
   Init_Stream_Struct(atom_file_name, file, prop, fct_getc, fct_putc,
 		     fct_flush, fct_close, fct_tell, fct_seek, fct_clearerr,
 		     pstm);
 
-  Set_Stream_Buffering(stm);
-
   return stm;
 }
 
@@ -400,9 +446,12 @@ Delete_Stream(int stm)
 {
   Del_Aliases_Of_Stream(stm);
 
-  stm_tbl[stm].file = 0;
+  Update_Mirrors_To_Del_Stream(stm);
 
-  while(stm_tbl[stm_last_used].file == 0)
+  Free(stm_tbl[stm]);
+  stm_tbl[stm] = NULL;
+
+  while(stm_tbl[stm_last_used] == NULL)
     stm_last_used--;
 }
 
@@ -418,14 +467,14 @@ Find_Free_Stream(void)
 {
   int stm;
 
-  for (stm = 0; stm < MAX_STREAM; stm++)
-    if (stm_tbl[stm].file == 0)
+  for (stm = 0; stm < stm_tbl_size; stm++)
+    if (stm_tbl[stm] == NULL)
       break;
 
-#ifndef FOR_EXTERNAL_USE
-  if (stm >= MAX_STREAM)
-    Pl_Err_Resource(resource_too_many_open_streams);
-#endif /* !FOR_EXTERNAL_USE */
+  if (stm == stm_tbl_size)
+    Extend_Array((char **) &stm_tbl, &stm_tbl_size, sizeof(StmInf *), TRUE);
+
+  stm_tbl[stm] = (StmInf *) Malloc(sizeof(StmInf));
 
   if (stm > stm_last_used)
     stm_last_used = stm;
@@ -517,6 +566,145 @@ Del_Aliases_Of_Stream(int stm)
 
 
 
+/*-------------------------------------------------------------------------*
+ * ADD_MIRROR_TO_STREAM                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Add_Mirror_To_Stream(int stm, int m_stm)
+{
+  StmInf *pstm = stm_tbl[stm];
+  StmInf *m_pstm = stm_tbl[m_stm];
+  StmLst *m;
+  
+  if (stm == m_stm)
+    return;
+
+  for(m = pstm->mirror; m ; m = m->next)
+    if (m->stm == m_stm)	/* already present */
+      return;
+
+  m = (StmLst *) Malloc(sizeof(StmLst));
+  m->stm = m_stm;
+  m->next = pstm->mirror;
+  pstm->mirror = m;
+
+  m = (StmLst *) Malloc(sizeof(StmLst));
+  m->stm = stm;
+  m->next = m_pstm->mirror_of;
+
+  m_pstm->mirror_of = m;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * DEL_MIRROR_FROM_STREAM                                                  *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Del_Mirror_From_Stream(int stm, int m_stm)
+{
+  StmInf *pstm = stm_tbl[stm];
+  StmInf *m_pstm = stm_tbl[m_stm];
+
+  if (!Remove_In_Stream_List(m_stm, &pstm->mirror))
+    return FALSE;		/* not found */
+
+  Remove_In_Stream_List(stm, &m_pstm->mirror_of);
+  return TRUE;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * UPDATE_MIRRORS_TO_DEL_STREAM                                            *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static void
+Update_Mirrors_To_Del_Stream(int stm)
+{
+  StmInf *pstm = stm_tbl[stm];
+  StmInf *m_pstm;
+  StmLst *m, *m1;
+  
+  m = pstm->mirror;
+  while(m)
+    {
+      m1 = m;
+      m_pstm = stm_tbl[m->stm];
+      m = m->next;
+      Free(m1);
+      Remove_In_Stream_List(stm, &m_pstm->mirror_of);
+    }
+  
+  m = pstm->mirror_of;
+  while(m)
+    {
+      m1 = m;
+      m_pstm = stm_tbl[m->stm];
+      m = m->next;
+      Free(m1);
+      Remove_In_Stream_List(stm, &m_pstm->mirror);
+    }
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * REMOVE_IN_STREAM_LIST                                                   *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static Bool
+Remove_In_Stream_List(int stm, StmLst **p_start)
+{
+  StmLst *m;
+
+  for(;;)
+    {
+      m = *p_start;
+
+      if (m == NULL)
+	break;
+
+      if (m->stm == stm)	/* found */
+	{
+	  *p_start = m->next;
+	  Free(m);
+	  return TRUE;
+	}
+
+      p_start = &m->next;
+    }
+
+  return FALSE;			/* not found */
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * FIND_STREAM_FROM_PSTM                                                   *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Find_Stream_From_PStm(StmInf *pstm)
+{
+  int stm;
+
+  for (stm = 0; stm <= stm_last_used; stm++)
+    if (stm_tbl[stm] == pstm)
+      return stm;
+
+  return -1;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * FLUSH_ALL_STREAMS                                                       *
  *                                                                         *
@@ -526,9 +714,9 @@ Flush_All_Streams(void)
 {
   int stm;
 
-  for (stm = 0; stm < MAX_STREAM; stm++)
-    if (stm_tbl[stm].file != 0)
-      Stream_Flush(stm_tbl + stm);
+  for (stm = 0; stm <= stm_last_used ; stm++)
+    if (stm_tbl[stm])
+      Stream_Flush(stm_tbl[stm]);
 }
 
 
@@ -541,52 +729,28 @@ Flush_All_Streams(void)
 void
 Set_Stream_Buffering(int stm)
 {
-  StmInf *pstm = stm_tbl + stm;
-  int buff_flag;
+  StmInf *pstm = stm_tbl[stm];
   FILE *f;
 
-  f = File_Star_Of_Stream(stm);
+  f = Stdio_Desc_Of_Stream(stm);
   if (f == NULL)
     {
       pstm->prop.buffering = STREAM_BUFFERING_NONE;
       return;
     }
 
-  switch (pstm->prop.buffering)
-    {
-    case STREAM_BUFFERING_NONE:
-      buff_flag = _IONBF;
-      break;
-
-    case STREAM_BUFFERING_LINE:
-      buff_flag = _IOLBF;
-#ifdef M_ix86_win32
-      if (!use_gui)		/* in Win32 console app, line buff = full */
-	buff_flag = _IONBF;	/* I prefer no buffering */
-#endif
-      break;
-
-    case STREAM_BUFFERING_BLOCK:
-      buff_flag = _IOFBF;
-      break;
-    }
-
-  /* for those architectures we cannot modify */
-  /* buffering when an I/O as already occured */
-#if defined(M_ix86_bsd) || defined(M_ix86_cygwin)
-  return;
-#endif
-
 #ifndef NO_USE_LINEDIT		/* if use_gui == 1 */
   if (pstm->file == (long) stdout && le_hook_set_line_buffering)
-    (*le_hook_set_line_buffering)(buff_flag != _IONBF);
+    (*le_hook_set_line_buffering)(pstm->prop.buffering != STREAM_BUFFERING_NONE);
   else
 #endif
-    setvbuf(f, NULL, buff_flag, BUFSIZ);
+
+    Stdio_Set_Buffering(f, pstm->prop.buffering);
 }
 
 
 
+
 #ifndef FOR_EXTERNAL_USE
 
 /*-------------------------------------------------------------------------*
@@ -608,8 +772,7 @@ Get_Stream_Or_Alias(WamWord sora_word, int test)
   if (tag_mask == TAG_ATM_MASK)	/* alias ? */
     {
       atom = UnTag_ATM(word);
-      stm = (atom == atom_glob_stream_alias) ? MAX_STREAM
-	: Find_Stream_By_Alias(atom);
+      stm = Find_Stream_By_Alias(atom);
       goto next_test;
     }
 
@@ -619,7 +782,8 @@ Get_Stream_Or_Alias(WamWord sora_word, int test)
       DEREF(Arg(stc_adr, 0), word, tag_mask1);
       stm = UnTag_INT(word);
 
-      if (Functor_And_Arity(stc_adr) == stream_1 && tag_mask1 == TAG_INT_MASK)
+      if (Functor_And_Arity(stc_adr) == stream_1 && 
+	  tag_mask1 == TAG_INT_MASK)
 	goto next_test;
     }
 
@@ -628,11 +792,9 @@ Get_Stream_Or_Alias(WamWord sora_word, int test)
 
   Pl_Err_Domain(domain_stream_or_alias, sora_word);
 
-
  next_test:
 
-  if (stm < 0 || stm > MAX_STREAM || stm_tbl[stm].file == 0 ||
-      (stm == MAX_STREAM && glob_str_stream.ptr == NULL))	/* global not in use */
+  if ((unsigned) stm > stm_last_used || stm_tbl[stm] == NULL)
     {
       if (test == STREAM_CHECK_VALID)
 	return -1;
@@ -645,14 +807,14 @@ Get_Stream_Or_Alias(WamWord sora_word, int test)
 
   if (test == STREAM_CHECK_INPUT)
     {
-      if (stm_tbl[stm].prop.input)
+      if (stm_tbl[stm]->prop.input)
 	goto ok;
 
       perm_oper = permission_operation_input;
     }
   else				/* test == STREAM_CHECK_OUTPUT */
     {
-      if (stm_tbl[stm].prop.output)
+      if (stm_tbl[stm]->prop.output)
 	goto ok;
 
       perm_oper = permission_operation_output;
@@ -681,14 +843,14 @@ Check_Stream_Type(int stm, Bool check_text, Bool for_input)
 
   if (check_text)
     {
-      if (stm_tbl[stm].prop.text)
+      if (stm_tbl[stm]->prop.text)
 	return;
 
       perm_type = permission_type_binary_stream;
     }
   else				/* check binary */
     {
-      if (!stm_tbl[stm].prop.text)
+      if (!stm_tbl[stm]->prop.text)
 	return;
 
       perm_type = permission_type_text_stream;
@@ -715,34 +877,81 @@ Check_Stream_Type(int stm, Bool check_text, Bool for_input)
 
 
 /*-------------------------------------------------------------------------*
- * FILE_NUMBER_OF_STREAM                                                   *
+ * MAKE_STREAM_TAGGED_WORD                                                 *
  *                                                                         *
- * return the fileno of a stream or -1 if this stream has not a fileno.    *
  *-------------------------------------------------------------------------*/
-int
-File_Number_Of_Stream(int stm)
+WamWord
+Make_Stream_Tagged_Word(int stm)
 {
-  FILE *f;
+  static WamWord h[2];
 
-  f = File_Star_Of_Stream(stm);
-  if (f)
-    return fileno(f);
+  h[0] = stream_1;
+  h[1] = Tag_INT(stm);
 
-  return -1;
+  return Tag_STC(h);
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * FILE_STAR_OF_STREAM                                                     *
+ * STDIO_IS_REPOSITIONABLE                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Stdio_Is_Repositionable(FILE *f)
+{
+  int fd = fileno(f);
+
+  return !isatty(fd) && lseek(fd, 0, SEEK_CUR) >= 0;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * STDIO_SET_BUFFERING                                                     *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Stdio_Set_Buffering(FILE *f, int buffering)
+{
+  int buff_flag;
+
+  switch (buffering)
+    {
+    case STREAM_BUFFERING_NONE:
+      buff_flag = _IONBF;
+      break;
+
+    case STREAM_BUFFERING_LINE:
+      buff_flag = _IOLBF;
+#ifdef M_ix86_win32
+      if (!use_gui)		/* in Win32 console app, line buff = full */
+	buff_flag = _IONBF;	/* I prefer no buffering */
+#endif
+      break;
+
+    case STREAM_BUFFERING_BLOCK:
+      buff_flag = _IOFBF;
+      break;
+    }
+  
+  setvbuf(f, NULL, buff_flag, BUFSIZ);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * STDIO_DESC_OF_STREAM                                                    *
  *                                                                         *
  * return a FILE * of a stream or NULL if it is not a stdio stream.        *
  *-------------------------------------------------------------------------*/
 FILE *
-File_Star_Of_Stream(int stm)
+Stdio_Desc_Of_Stream(int stm)
 {
-  StmInf *pstm = stm_tbl + stm;
+  StmInf *pstm = stm_tbl[stm];
 
   if (stm == stm_stdin)		/* works also for stdin with linedit */
     return stdin;
@@ -756,6 +965,26 @@ File_Star_Of_Stream(int stm)
 
 
 
+/*-------------------------------------------------------------------------*
+ * IO_FILENO_OF_STREAM                                                     *
+ *                                                                         *
+ * return the fileno of a stream or -1 if this stream has not a fileno.    *
+ *-------------------------------------------------------------------------*/
+int
+Io_Fileno_Of_Stream(int stm)
+{
+  FILE *f;
+
+  f = Stdio_Desc_Of_Stream(stm);
+  if (f)
+    return fileno(f);
+
+  return -1;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * The following functions replaces standard fgetc/... on stdin if a TTY.  *
  * It uses linedit to provide a more comfortable interface.                *
@@ -801,7 +1030,9 @@ TTY_Getc(void)
 
 				/* tty_ptr must remain NULL for reentrancy */
       SAVE_FOR_REENTRANCY;
-      tty_buff = LE_FGets(tty_buff, TTY_BUFFER_SIZE, le_prompt, 1);
+      tty_buff = LE_FGets(tty_buff, TTY_BUFFER_SIZE, le_prompt, 
+			  use_le_prompt);
+      use_le_prompt = 0;
       RESTORE_FOR_REENTRANCY;
       tty_linedit_depth--;
 
@@ -817,7 +1048,7 @@ TTY_Getc(void)
 
       tty_ptr = tty_buff;
     				/* simulate the echo (+ '\n') on output */
-      pstm = stm_tbl + stm_stdout;
+      pstm = stm_tbl[stm_stdout];
       pstm->char_count += strlen(tty_buff);
       pstm->line_count++;
       pstm->line_pos = 0;
@@ -866,6 +1097,9 @@ TTY_Get_Key(Bool echo, Bool catch_ctrl_c)
   c = LE_Get_Key(echo, catch_ctrl_c);
   RESTORE_FOR_REENTRANCY;
 
+  if (LE_Interrupted_By_Ctrl_C(c))
+    Execute_A_Continuation((CodePtr) LE_Get_Ctrl_C_Return_Value());
+
   return c;
 }
 
@@ -910,7 +1144,7 @@ TTY_Clearerr(void)
       if (pstm->prop.eof_action == STREAM_EOF_ACTION_EOF_CODE)		    \
         return EOF;							    \
  									    \
-      /* here: eof_action==STREAM_EOF_ACTION_RESET */			    \
+      /* here: eof_action == STREAM_EOF_ACTION_RESET */			    \
       pstm->eof_reached = FALSE;					    \
       if (pstm->prop.reposition)					    \
         Stream_Set_Position(pstm, SEEK_SET, 0, 0, 0, 0);		    \
@@ -939,21 +1173,25 @@ TTY_Clearerr(void)
 /*-------------------------------------------------------------------------*
  * BASIC_CALL_FCT_GETC                                                     *
  *                                                                         *
- * only useful if !defined(NO_USE_PIPED_STDIN_FOR_CONSULT)                 *
  *-------------------------------------------------------------------------*/
 static int
 Basic_Call_Fct_Getc(StmInf *pstm)
 {
   int c;
+  StmLst *m;
 #ifndef NO_USE_PIPED_STDIN_FOR_CONSULT
   if (SYS_VAR_SAY_GETC && pstm->file == (long) stdin)
-    /* could also test pstm->fct_getc == fgetc) */
+    /* could also test pstm->fct_getc == fgetc */
     {
       putchar(CHAR_TO_EMIT_WHEN_CHAR);
       fflush(stdout);
     }
 #endif
   c = (*pstm->fct_getc) (pstm->file);
+
+  if (c != EOF)
+    for (m = pstm->mirror; m ; m = m->next)
+      Stream_Putc(c, stm_tbl[m->stm]);
  
   return c;
 }
@@ -961,6 +1199,24 @@ Basic_Call_Fct_Getc(StmInf *pstm)
 
 
 
+/*-------------------------------------------------------------------------*
+ * BASIC_CALL_FCT_PUTC                                                     *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static void
+Basic_Call_Fct_Putc(int c, StmInf *pstm)
+{
+  StmLst *m;
+
+  (*pstm->fct_putc) (c, pstm->file);
+
+  for (m = pstm->mirror; m ; m = m->next)
+    Stream_Putc(c, stm_tbl[m->stm]);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * STREAM_GET_KEY                                                          *
  *                                                                         *
@@ -973,7 +1229,7 @@ Stream_Get_Key(StmInf *pstm, int echo, int catch_ctrl_c)
   Bool simulate;
 
 #ifndef NO_USE_LINEDIT
-  if (pstm - stm_tbl == stm_stdin) /* the stdin stream used with linedit */
+  if (pstm == stm_tbl[stm_stdin]) /* the stdin stream used with linedit */
     simulate = FALSE;
   else
 #endif
@@ -1003,7 +1259,7 @@ Stream_Get_Key(StmInf *pstm, int echo, int catch_ctrl_c)
       while (Basic_Call_Fct_Getc(pstm) >= ' ')
 	;
 
-      Update_Counters((stm_tbl + stm_stdout), '\n'); /* reflect \n */
+      Update_Counters(stm_tbl[stm_stdout], '\n'); /* reflect \n */
     }
 
   if (c == EOF)
@@ -1162,8 +1418,11 @@ Stream_Gets_Prompt(char *prompt, StmInf *pstm_o,
 {
 #ifndef NO_USE_LINEDIT
   char *save_le_prompt = le_prompt;
+  int save_use_le_prompt = use_le_prompt;
   le_prompt = prompt;
 
+  use_le_prompt = 1;
+
   if (pstm_i->fct_getc != TTY_Getc)
 #endif
     Stream_Printf(pstm_o, prompt);
@@ -1171,6 +1430,7 @@ Stream_Gets_Prompt(char *prompt, StmInf *pstm_o,
   str = Stream_Gets(str, size, pstm_i);
 
 #ifndef NO_USE_LINEDIT
+  use_le_prompt = save_use_le_prompt;
   le_prompt = save_le_prompt;
 #endif
   return str;
@@ -1186,10 +1446,7 @@ Stream_Gets_Prompt(char *prompt, StmInf *pstm_o,
 void
 Stream_Putc(int c, StmInf *pstm)
 {
-  long file = pstm->file;
-
-  (*pstm->fct_putc) (c, file);
-
+  Basic_Call_Fct_Putc(c, pstm);
   Update_Counters(pstm, c);
 }
 
@@ -1203,14 +1460,13 @@ Stream_Putc(int c, StmInf *pstm)
 int
 Stream_Puts(char *str, StmInf *pstm)
 {
-  long file = pstm->file;
   char *p;
   int c;
 
   for (p = str; *p; p++)
     {
       c = *p;
-      (*pstm->fct_putc) (c, file);	/* like Stream_Putc */
+      Basic_Call_Fct_Putc(c, pstm); /* like Stream_Putc */
       Update_Counters(pstm, c);
     }
 
@@ -1227,7 +1483,6 @@ Stream_Puts(char *str, StmInf *pstm)
 int
 Stream_Printf(StmInf *pstm, char *format, ...)
 {
-  long file = pstm->file;
   va_list arg_ptr;
   static char str[BIG_BUFFER];
   char *p;
@@ -1241,7 +1496,7 @@ Stream_Printf(StmInf *pstm, char *format, ...)
   for (p = str; *p; p++)
     {
       c = *p;
-      (*pstm->fct_putc) (c, file);	/* like Stream_Putc */
+      Basic_Call_Fct_Putc(c, pstm); /* like Stream_Putc */
       Update_Counters(pstm, c);
     }
 
@@ -1477,38 +1732,38 @@ err:
 
 /*-------------------------------------------------------------------------*
  * The following functions allows the user to handle streams on C strings  *
- * Any stream can be a string stream. There is a global string stream used *
- * by predicates like write_to_atom/2,...                                  *
+ * Any stream can be a string stream. To avoid unnecessary malloc/free, we *
+ * use as long as possible 2 str stream statically allocated (1 for input, *
+ * 1 for output). This optimizes the use of preds like write_to_atom/2,... *
+ * NB: The buff of the output static str stream is reused (no free on it). *
+ * A dynamic str stream is allocated when it is not possible to use static *
+ * ones. Such a str stream is freed at the close.                          *
  *-------------------------------------------------------------------------*/
 
 
 /*-------------------------------------------------------------------------*
  * ADD_STR_STREAM                                                          *
  *                                                                         *
- * buff==NULL <=> write mode <=> str_stream->buff_alloc_size!=0            *
+ * buff == NULL means output stream mode (str_stream->buff_alloc_size != 0)*
  *-------------------------------------------------------------------------*/
 int
-Add_Str_Stream(Bool use_global, char *buff)
+Add_Str_Stream(char *buff, int prop_other)
 {
   int stm;
   StmInf *pstm;
   StmProp prop;
   StrSInf *str_stream;
 
-  if (use_global)
+  str_stream = (buff) ? &static_str_stream_rd : &static_str_stream_wr;
+  if (str_stream->ptr != NULL)	/* in use ? */
     {
-      str_stream = &glob_str_stream;
-      if (buff && str_stream->buff_alloc_size != 0)
-	Free(str_stream->buff);
+      str_stream = (StrSInf *) Malloc(sizeof(StrSInf));
+      str_stream->buff_alloc_size = 0;
     }
-  else
-    str_stream = (StrSInf *) Malloc(sizeof(StrSInf));
 
   if (buff)
     {
       str_stream->buff = buff;
-      str_stream->ptr = buff;
-      str_stream->buff_alloc_size = 0;
 
       prop.mode = STREAM_MODE_READ;
       prop.input = TRUE;
@@ -1516,48 +1771,34 @@ Add_Str_Stream(Bool use_global, char *buff)
     }
   else
     {
-      if (!use_global || str_stream->buff_alloc_size == 0)
-	str_stream->buff = (char *) Malloc(STR_STREAM_WRITE_BLOCK);
-
-      str_stream->ptr = str_stream->buff;
-      str_stream->buff_alloc_size = STR_STREAM_WRITE_BLOCK;
+      if (str_stream->buff_alloc_size == 0)
+	{
+	  str_stream->buff = (char *) Malloc(STR_STREAM_WRITE_BLOCK);
+	  str_stream->buff_alloc_size = STR_STREAM_WRITE_BLOCK;
+	}
 
       prop.mode = STREAM_MODE_WRITE;
       prop.input = FALSE;
       prop.output = TRUE;
     }
 
+  str_stream->ptr = str_stream->buff; /* ptr != NULL <=> in use for global */
+
   prop.text = 1;
   prop.reposition = FALSE;
   prop.buffering = STREAM_BUFFERING_NONE;
   prop.eof_action = STREAM_EOF_ACTION_EOF_CODE;
   prop.special_close = TRUE;
-  prop.other = 0;
+  prop.other = prop_other;
 
-  stm = (use_global) ? MAX_STREAM : Find_Free_Stream();
-  pstm = stm_tbl + stm;
+  stm = Find_Free_Stream();
+  pstm = stm_tbl[stm];
 
-  pstm->atom_file_name = atom_void;
-
-  pstm->file = (long) str_stream;
-  pstm->prop = prop;
-
-  pstm->fct_getc = (StmFct) Str_Stream_Getc;
-  pstm->fct_putc = (StmFct) Str_Stream_Putc;
-  pstm->fct_flush = STREAM_FCT_UNDEFINED;
-  pstm->fct_close = STREAM_FCT_UNDEFINED;
-  pstm->fct_tell = STREAM_FCT_UNDEFINED;
-  pstm->fct_seek = STREAM_FCT_UNDEFINED;
-  pstm->fct_clearerr = STREAM_FCT_UNDEFINED;
-
-  pstm->eof_reached = FALSE;
-
-  PB_Init(pstm->pb_char);
-
-  pstm->char_count = 0;
-  pstm->line_count = 0;
-  pstm->line_pos = 0;
-  PB_Init(pstm->pb_line_pos);
+  Init_Stream_Struct(atom_constant_term_stream, (long) str_stream, prop,
+		     (StmFct) Str_Stream_Getc, (StmFct) Str_Stream_Putc,
+		     STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED, 
+		     STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
+		     STREAM_FCT_UNDEFINED, pstm);
 
   return stm;
 }
@@ -1572,18 +1813,19 @@ Add_Str_Stream(Bool use_global, char *buff)
 void
 Delete_Str_Stream(int stm)
 {
-  StrSInf *str_stream;
+  StrSInf *str_stream = (StrSInf *) (stm_tbl[stm]->file);
 
-  if (stm == MAX_STREAM)	/* the global str stream */
+  if (str_stream == &static_str_stream_rd ||
+      str_stream == &static_str_stream_wr)
     {
-      glob_str_stream.ptr = NULL;	/* ie. global stream not in use */
-      return;
+      str_stream->ptr = NULL;	/* not in use */
+    }
+  else
+    {
+      if (str_stream->buff_alloc_size)
+	Free(str_stream->buff);
+      Free(str_stream);
     }
-
-  str_stream = (StrSInf *) (stm_tbl[stm].file);
-  if (str_stream->buff_alloc_size)
-    Free(str_stream->buff);
-  Free(str_stream);
 
   Delete_Stream(stm);
 }
@@ -1601,7 +1843,7 @@ Term_Write_Str_Stream(int stm)
 {
   StrSInf *str_stream;
 
-  str_stream = (StrSInf *) (stm_tbl[stm].file);
+  str_stream = (StrSInf *) (stm_tbl[stm]->file);
   *(str_stream->ptr) = '\0';
 
   return str_stream->buff;
@@ -1654,4 +1896,3 @@ Str_Stream_Putc(int c, StrSInf *str_stream)
 
   *(str_stream->ptr)++ = c;
 }
-
