@@ -6,7 +6,7 @@
  * Descr.: FD instruction implementation                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_inst.c,v 1.7 2001/12/07 21:05:10 diaz Exp $ */
+/* $Id: fd_inst.c,v 1.10 2003/01/06 12:18:10 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -1367,6 +1367,6 @@ Fd_Variable_To_String0(WamWord *fdv_adr)
 void
 Fd_Display_Extra_Cstr(WamWord *fdv_adr)
 {
-  Stream_Printf(stm_tbl + stm_stdout, MSG_VECTOR_TOO_SMALL,
+  Stream_Printf(stm_tbl[stm_stdout], MSG_VECTOR_TOO_SMALL,
 		Cstr_Offset(fdv_adr));
 }
