@@ -6,7 +6,7 @@
  * Descr.: machine dependent features                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.c,v 1.8 2001/12/20 20:50:58 diaz Exp $ */
+/* $Id: machine.c,v 1.17 2003/01/06 12:18:11 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,6 +35,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "gp_config.h"		/* ensure __unix__ defined if not Win32 */
+
 #if defined(_WIN32) || defined(__CYGWIN__)
 #include <windows.h>
 #endif
@@ -124,9 +126,11 @@ static char cur_work_dir[MAXPATHLEN];
 
 #ifdef M_USE_MAGIC_NB_TO_DETECT_STACK_NAME
 
-static WamWord *check_adr[NB_OF_STACKS + 1];
+static WamWord *check_adr[NB_OF_STACKS];
 
-#else
+#endif
+
+#ifdef M_USE_MMAP
 
 static int page_size;
 
@@ -274,25 +278,26 @@ M_Allocate_Stacks(void)
 
 #if !defined(M_ix86_win32) && defined(M_MMAP_HIGH_ADR_ALT)
   i = 0;
-try_mmap:
+ try_mmap:
 #endif
 #ifdef DEBUG
   DBGPRINTF("trying at high addr:%lx\n", (long) addr);
 #endif
   addr = (WamWord *) Round_Down((long) addr, getpagesize());
   addr -= len;
-
 #ifdef M_ix86_win32
   addr = (WamWord *) VirtualAlloc(addr, len,
 				  MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
 #else
   addr = (WamWord *) mmap((caddr_t) addr, len, PROT_READ | PROT_WRITE,
+			  MAP_PRIVATE
+#ifdef MMAP_NEEDS_FIXED
+			  | MAP_FIXED
+#endif
 #ifdef MAP_ANON
-			  MAP_PRIVATE | MAP_ANON, -1,
-#elif defined(M_sparc_sunos) || defined(M_sparc_solaris)
-			  MAP_PRIVATE | MAP_FIXED, fd,
+			  | MAP_ANON, -1,
 #else
-			  MAP_PRIVATE, fd,
+			  , fd,
 #endif
 			  0);
 #endif
@@ -303,11 +308,14 @@ try_mmap:
 #endif
 
 #ifdef M_ix86_win32
-  if (addr == NULL)
+  if (addr == NULL
 #else
   if ((long) addr == -1
-      || ((long) addr & (((1L << TAG_SIZE) - 1) << VALUE_SIZE)) != 0)
 #endif
+#if TAG_SIZE_HIGH > 0
+      || (((unsigned long) (addr) + len) >> (WORD_SIZE - TAG_SIZE_HIGH)) != 0
+#endif
+      )
     {
 #if !defined(M_ix86_win32) && defined(M_MMAP_HIGH_ADR_ALT)
       if (i == 0)
@@ -347,7 +355,7 @@ try_mmap:
 #endif
 
 #if defined(M_sparc_solaris) || defined(M_ix86_solaris) || \
-    defined(M_ix86_sco)
+    defined(M_ix86_sco) || defined(M_x86_64_linux)
   {
     struct sigaction act;
 
@@ -382,11 +390,15 @@ Fill_Magic_Adr_Table(void)
 
   for (i = 0; i < NB_OF_STACKS; i++)
     {
-      check_adr[i] = stk_tbl[i].stack + stk_tbl[i].size - M_SECURITY_MARGIN;
-      *check_adr[i] = M_MAGIC;
+      if (stk_tbl[i].size == 0)
+	check_adr[i] = (WamWord *) NULL;
+      else
+	{
+	  check_adr[i] = stk_tbl[i].stack + stk_tbl[i].size -
+	    M_SECURITY_MARGIN;
+	  *check_adr[i] = M_MAGIC;
+	}
     }
-
-  check_adr[NB_OF_STACKS] = NULL;
 }
 
 #endif
@@ -482,6 +494,10 @@ SIGSEGV_Handler(void)
 
 void
 SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
+#elif defined(M_x86_64_linux)
+
+void
+SIGSEGV_Handler(int sig, siginfo_t *sip, void *scp)
 #else
 
 static void
@@ -512,6 +528,10 @@ SIGSEGV_Handler(int sig)
 
   WamWord *addr = (WamWord *) scp.cr2;
 
+#elif defined(M_x86_64_linux)
+
+  WamWord *addr = (WamWord *) sip->si_addr;
+
 #elif defined(M_ix86_sco)
 
   WamWord *addr = (WamWord *) si->si_addr;
@@ -569,11 +589,11 @@ SIGSEGV_Handler(int sig)
 void
 M_Check_Magic_Words(void)
 {
-  WamWord **p = (WamWord **) check_adr;
+  int i;
 
-  for (p = (WamWord **) check_adr; *p; p++)
-    if (**p != M_MAGIC)
-      Stack_Overflow(p - (WamWord **) check_adr);
+  for (i = 0; i < NB_OF_STACKS; i++)
+    if (check_adr[i] && *check_adr[i] != M_MAGIC)
+      Stack_Overflow(i);
 }
 
 #endif
@@ -774,8 +794,18 @@ M_Real_Time(void)
 void
 M_Randomize(void)
 {
-  M_Set_Seed(M_Real_Time());
-  M_Set_Seed(M_Random_Integer(256));
+#ifdef M_ix86_win32
+  int seed = GetTickCount();
+#else
+  struct timeval tv;
+  int seed;
+
+  gettimeofday(&tv, NULL);
+  seed = ((tv.tv_sec * 1000) + (tv.tv_usec / 1000));
+#endif
+  seed = (seed ^ getpid()) & 0xFFFFFF;
+
+  M_Set_Seed(seed);
 }
 
 
