@@ -6,7 +6,7 @@
  * Descr.: WAM instruction implementation                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_inst.c,v 1.7 2001/12/20 20:50:58 diaz Exp $ */
+/* $Id: wam_inst.c,v 1.10 2003/01/06 12:18:12 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -436,12 +436,8 @@ Put_Unsafe_Value(WamWord start_word)
       return res_word;
     }
 
-#if 0
   Do_Copy_Of_Word(tag_mask, word);
   return word;
-#else
-  return start_word;
-#endif
 }
 
 
@@ -875,6 +871,30 @@ Unify_Structure(int func, int arity)
 
 
 
+/*-------------------------------------------------------------------------*
+ * GLOBALIZE_IF_IN_LOCAL                                                   *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+WamWord
+Globalize_If_In_Local(WamWord start_word)
+{
+  WamWord word, tag_mask;
+  WamWord *adr;
+
+  DEREF(start_word, word, tag_mask);
+  if (tag_mask == TAG_REF_MASK)
+    {
+      adr = UnTag_REF(word);
+      if (Is_A_Local_Adr(adr))
+	Globalize_Local_Unbound_Var(adr, start_word);
+    }
+
+  return start_word;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * ALLOCATE                                                                *
  *                                                                         *
