@@ -6,7 +6,7 @@
  * Descr.: line editor                                                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: linedit.c,v 1.5 2001/12/07 21:02:58 diaz Exp $ */
+/* $Id: linedit.c,v 1.10 2003/01/06 12:18:12 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,6 +33,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "../EnginePl/gp_config.h"
+
 #define LE_DEFINE_HOOK_MACROS
 
 #include "terminal.h"
@@ -51,13 +53,30 @@
 #endif
 
 
+#if 1
+#define TREAT_BUFFERED_CHARS	/* treat buffered chars at start (X paste) */
+#endif
+
+#if 1
+#define NO_DUP_IN_HISTORY	/* do not put in history line == the last */
+#endif
+
+#if 1
+#define IGNORE_QUOTED_PART	/* ingore quoted item in bracket matching */
+#endif
+
+#if 0
+#define NO_USE_SELECT		/* no use select(2) for temporisation */
+#endif
+
+
 
 
 /*---------------------------------*
  * Constants                       *
  *---------------------------------*/
 
-#define LINEDIT_VERSION            "2.4"
+#define LINEDIT_VERSION            "2.5"
 
 #define MAX_HISTORY_LINES          64
 #define MAX_SEPARATORS             256
@@ -68,14 +87,15 @@
 
 #define NB_MATCH_LINES_BEFORE_ASK  20
 
+#define OPEN_BRACKET               "([{"
+#define CLOSE_BRACKET              ")]}"
+
 #ifdef NO_USE_SELECT
 #define BRACKET_TIMMING            300000	/* in microseconds */
 #else
 #define BRACKET_TIMMING            900000	/* in microseconds */
 #endif
 
-#define IGNORE_QUOTED_PART
-
 
 
 
@@ -148,6 +168,8 @@ static char *Skip(char *from, char *limit, int res_sep_cmp, int direction);
 
 static int Is_A_Separator(char c);
 
+static int Tab_To_Spaces(int p);
+
 
 
 static void History_Add_Line(char *line, int length);
@@ -226,8 +248,7 @@ LE_Gets(char *str)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 char *
-LE_FGets(char *str, int size, char *prompt,
-	 int display_prompt)
+LE_FGets(char *str, int size, char *prompt, int display_prompt)
 {
   char *pos = str;
   char *end = str;
@@ -239,8 +260,6 @@ LE_FGets(char *str, int size, char *prompt,
   int h_no = Hist_End_Entry();
   int rest_length;
   int tab_count = 0;
-  char *open_bracket = "([{";
-  char *close_bracket = ")]}";
   int count_bracket[3];
 
   LE_Initialize();
@@ -252,7 +271,7 @@ LE_FGets(char *str, int size, char *prompt,
   LE_Open_Terminal();
   global_str = str;
 
-#if 1				/* treat buffered lines (for paste) */
+#ifdef TREAT_BUFFERED_CHARS	/* treat buffered lines (for paste) */
   while (KBD_IS_NOT_EMPTY)
     {
       if (end - str >= size || ((c = LE_Get_Char()) == '\n') ||
@@ -262,7 +281,7 @@ LE_FGets(char *str, int size, char *prompt,
 	  goto return_is_read;
 	}
       if (c == '\t')		/* '\t' on output would cause trouble */
-	for (n = 0; n < 8; n++)
+	for(n = Tab_To_Spaces(end - str); n; n--)
 	  *end++ = ' ';
       else
 	*end++ = c;
@@ -291,6 +310,8 @@ LE_FGets(char *str, int size, char *prompt,
 	{			/* save global vars for reentrancy */
 	  int save_prompt_length = prompt_length;
 
+	  FORWD(end - pos, pos); /* go to EOL to avoid multi-line */
+				/* truncation on the output */
 	  LE_Close_Terminal();
 	  c = *end;
 	  *end = '\0';		/* to allow the handler to use/test str */
@@ -471,7 +492,7 @@ LE_FGets(char *str, int size, char *prompt,
 
 	case KEY_ESC('B'):	/* go to previous word */
 	case KEY_CTRL_EXT_LEFT:
-	  p = (pos == str) ? pos : pos - 1;	/* to avoid start of a word */
+	  p = (pos == str) ? pos : pos - 1; /* to avoid start of a word */
 	  p = Skip(p, str, 1, -1);	/* skip separators */
 	  p = Skip(p, str, 0, -1);	/* skip non separators */
 	  p = Skip(p, end, 1, +1);	/* skip separators */
@@ -532,7 +553,7 @@ LE_FGets(char *str, int size, char *prompt,
 		}
 	      goto error;
 	    }
-	  p = (pos == str) ? pos : pos - 1;	/* to avoid start of a word */
+	  p = (pos == str) ? pos : pos - 1; /* to avoid start of a word */
 	  p = Skip(p, str, 0, -1);	/* skip non separators */
 	  p = Skip(p, end, 1, +1);	/* skip separators */
 	  w = *pos;		/* prefix from p to pos */
@@ -555,6 +576,13 @@ LE_FGets(char *str, int size, char *prompt,
 	  continue;
 
 
+	case KEY_ESC('\t'):	/* transform a tab to spaces */
+	  for (n = Tab_To_Spaces(pos - str); n; n--)
+	    if (!New_Char(' ', str, size, &pos, &end))
+	      goto error;
+	  continue;
+
+
 	case KEY_CTRL('V'):	/* switch insert mode (on/off) */
 	case KEY_EXT_INSERT:
 	  ins_mode = 1 - ins_mode;
@@ -576,6 +604,8 @@ LE_FGets(char *str, int size, char *prompt,
 	case '\n':
 	case '\r':
 	return_is_read:
+  	  FORWD(end - pos, pos); /* go to EOL to avoid multi-line */
+				/* truncation on the output */
 	  *end = '\0';
 	  History_Add_Line(str, end - str);
 	  if (end - str < size)	/* '\n' can be added */
@@ -691,10 +721,10 @@ LE_FGets(char *str, int size, char *prompt,
 	    goto error;
 	  /* brackets ([{ }]): matching */
 	  if (KBD_IS_NOT_EMPTY ||
-	      (q = strchr(close_bracket, c)) == NULL)
+	      (q = strchr(CLOSE_BRACKET, c)) == NULL)
 	    continue;
 
-	  n = q - close_bracket;
+	  n = q - CLOSE_BRACKET;
 
 	  count_bracket[0] = count_bracket[1] = count_bracket[2] = 0;
 	  count_bracket[n]--;
@@ -706,14 +736,14 @@ LE_FGets(char *str, int size, char *prompt,
 		goto bracket_exit;
 
 	      c = *p;
-	      if ((q = strchr(close_bracket, c)) != NULL)
+	      if ((q = strchr(CLOSE_BRACKET, c)) != NULL)
 		{
-		  count_bracket[q - close_bracket]--;
+		  count_bracket[q - CLOSE_BRACKET]--;
 		  continue;
 		}
 
-	      if ((q = strchr(open_bracket, c)) != NULL)
-		if (++count_bracket[q - open_bracket] > 0)
+	      if ((q = strchr(OPEN_BRACKET, c)) != NULL)
+		if (++count_bracket[q - OPEN_BRACKET] > 0)
 		  goto bracket_exit;
 
 #ifdef IGNORE_QUOTED_PART
@@ -953,6 +983,21 @@ Is_A_Separator(char c)
 
 
 
+/*-------------------------------------------------------------------------*
+ * TAB_TO_SPACES                                                           *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static int
+Tab_To_Spaces(int p)
+{
+  p += prompt_length;
+  p = 8 - (p % 8);
+  return p;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * HISTORY_ADD_LINE                                                        *
  *                                                                         *
@@ -967,6 +1012,11 @@ History_Add_Line(char *line, int length)
   if (*p == '\0')		/* do not add an empty line */
     return;
 
+#ifdef NO_DUP_IN_HISTORY
+  if (hist_end > 0 && strcmp(line, hist_tbl[hist_end - 1].line) == 0)
+    return;
+#endif
+
   History_Update_Line(line, length, hist_end);
 
   Hist_Inc(hist_end);
@@ -1308,7 +1358,7 @@ Display_Help(void)
 
   L("");
   sprintf(buff,
-	  "   linedit %-25s Copyright (C) 1999-2001 Daniel Diaz",
+	  "   linedit %-25s Copyright (C) 1999-2003 Daniel Diaz",
 	  LINEDIT_VERSION);
   L(buff);
   L("");
@@ -1335,8 +1385,8 @@ Display_Help(void)
   L("   Esc-W   copy selection            Ctl-Y   past selection");
   L("");
   L("                           Miscellaneous");
-  L("   Ctl-V   insert mode switch        Ctl-I   completion (twice=all)");
-  L("   Esc-?   display this help");
+  L("   Ctl-V   insert mode switch        Ctl-I   completion (twice = all)");
+  L("   Esc-?   display this help       Esc-Ctl-I insert spaces for tab");
   L("");
 }
 
@@ -1364,7 +1414,8 @@ start:
   if (catch_ctrl_c && LE_Is_Interrupt_Key(c))
     {
       LE_Close_Terminal();
-      Emit_Ctrl_C();
+      if ((ctrl_c_ret_val = Emit_Ctrl_C()) != 0)
+	return -2;
       goto start;
     }
 
