@@ -6,7 +6,7 @@
  * Descr.: translation file for Linux/OSF on alpha                         *
  * Author: Alexander Diemand, Daniel Diaz                                  *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: alpha_any.c,v 1.6 2001/12/20 20:50:58 diaz Exp $ */
+/* $Id: alpha_any.c,v 1.9 2003/01/06 12:18:12 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -677,11 +677,11 @@ Pl_Jump(char *label)
 
 
 /*-------------------------------------------------------------------------*
- * PL_CALL                                                                 *
+ * PREP_CP                                                                 *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Call(char *label)
+Prep_CP(void)
 {
 #ifdef MAP_REG_CP
   Inst_Printf("lda", "%s,$Lcont%d", asm_reg_cp, w_label);	/* CP = $Lcont%d */
@@ -689,15 +689,38 @@ Pl_Call(char *label)
   Inst_Printf("lda", "$4,$Lcont%d", w_label);	/* CP = $Lcont%d */
   Inst_Printf("stq", "$4,%s", asm_reg_cp);
 #endif
+}
 
-  Pl_Jump(label);
 
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
   Label_Printf("$Lcont%d:", w_label++);
 
   /* get GP back */
   /* this has only be introduced to get dynamic object loading work */
   /* Inst_Printf("ldgp","$gp,0($27)"); */
+}
 
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_CALL                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Call(char *label)
+{
+  Prep_CP();
+  Pl_Jump(label);
+  Here_CP();
 }
 
 
