@@ -6,7 +6,7 @@
  * Descr.: translation file for Linux/Cygwin/... on intel x86              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ix86_any.c,v 1.9 2001/12/20 20:50:58 diaz Exp $ */
+/* $Id: ix86_any.c,v 1.12 2003/01/06 12:18:12 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -187,7 +187,10 @@ Code_Start(char *label, int prolog, int global)
   Label(label);
 
   if (!prolog)
-    Inst_Printf("subl", "$%d,%%esp", MAX_C_ARGS_IN_C_CODE * 4);
+    {
+      Inst_Printf("pushl", "%%esi"); /* used if argno>FC_MAX_ARGS_IN_REGS */
+      Inst_Printf("subl", "$%d,%%esp", MAX_C_ARGS_IN_C_CODE * 4);
+    }
 }
 
 
@@ -250,6 +253,32 @@ Pl_Jump(char *label)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PREP_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Prep_CP(void)
+{
+  Inst_Printf("movl", "$.Lcont%d,%s", w_label, asm_reg_cp);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
+  Label_Printf(".Lcont%d:", w_label++);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CALL                                                                 *
  *                                                                         *
@@ -257,9 +286,9 @@ Pl_Jump(char *label)
 void
 Pl_Call(char *label)
 {
-  Inst_Printf("movl", "$.Lcont%d,%s", w_label, asm_reg_cp);
+  Prep_CP();
   Pl_Jump(label);
-  Label_Printf(".Lcont%d:", w_label++);
+  Here_CP();
 }
 
 
@@ -554,7 +583,8 @@ Call_C_Arg_Reg_X(int offset, int adr_of, int index)
   if (!r_eq_r_aux)
     Inst_Printf("movl", "%s,%s", r_aux, r);
 
- finish:  
+ finish:
+  ;    /* gcc3 does not like use of label at end of compound statement */
   AFTER_ARG;
 
   return 1;
@@ -825,6 +855,7 @@ void
 C_Ret(void)
 {
   Inst_Printf("addl", "$%d,%%esp", MAX_C_ARGS_IN_C_CODE * 4);
+  Inst_Printf("popl", "%%esi");
   Inst_Printf("ret", "");
 }
 
