@@ -6,7 +6,7 @@
  * Descr.: translation file for Win32 on intel x86 (for nasm)              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ix86_win32.c,v 1.7 2001/12/20 20:50:58 diaz Exp $ */
+/* $Id: ix86_win32.c,v 1.11 2003/01/06 12:18:12 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -255,6 +255,32 @@ Pl_Jump(char *label)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PREP_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Prep_CP(void)
+{
+  Inst_Printf("mov", "dword %s,.Lcont%d", asm_reg_cp, w_label);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
+  Label_Printf(".Lcont%d:", w_label++);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CALL                                                                 *
  *                                                                         *
@@ -264,9 +290,9 @@ Pl_Call(char *label)
 {
   BT_String_Add(&bt_symbol, strdup(label));
 
-  Inst_Printf("mov", "dword %s,.Lcont%d", asm_reg_cp, w_label);
+  Prep_CP();
   Pl_Jump(label);
-  Label_Printf(".Lcont%d:", w_label++);
+  Here_CP();
 }
 
 
