@@ -6,7 +6,7 @@
  * Descr.: translation file for SunOs/Solaris on sparc                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: sparc_any.c,v 1.7 2001/12/20 20:50:58 diaz Exp $ */
+/* $Id: sparc_any.c,v 1.10 2003/01/06 12:18:13 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -242,6 +242,34 @@ Pl_Jump(char *label)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PREP_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Prep_CP(void)
+{
+  fprintf(stderr, "PREP_CP not implemented\n");
+  exit(1);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
+  fprintf(stderr, "HERE_CP not implemented\n");
+  exit(1);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CALL                                                                 *
  *                                                                         *
@@ -380,21 +408,21 @@ Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
 
 #define MAX_ARGS_IN_REGS 6
 
-#define BEFORE_ARG                                                          \
-{                                                                           \
-  char r[4];                                                                \
-                                                                            \
-  if (offset < MAX_ARGS_IN_REGS)                                            \
-    sprintf(r, "%%o%d", offset);                                            \
-  else                                                                      \
+#define BEFORE_ARG				\
+{						\
+  char r[4];					\
+						\
+  if (offset < MAX_ARGS_IN_REGS)		\
+    sprintf(r, "%%o%d", offset);		\
+  else						\
     strcpy(r, "%l7");
 
 
-#define AFTER_ARG                                                           \
-  if (offset >= MAX_ARGS_IN_REGS)                                           \
-    Delay_Printf("st","%s,[%%sp+%d]", r,                                    \
-                 92 + (offset - MAX_ARGS_IN_REGS) * 4);                     \
- }
+#define AFTER_ARG					\
+  if (offset >= MAX_ARGS_IN_REGS)			\
+    Delay_Printf("st","%s,[%%sp+%d]", r,		\
+                 92 + (offset - MAX_ARGS_IN_REGS) * 4);	\
+}
 
 
 
