@@ -6,7 +6,7 @@
  * Descr.: parser                                                          *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2003 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_parser.c,v 1.6 2001/12/07 21:05:11 diaz Exp $ */
+/* $Id: wam_parser.c,v 1.9 2003/01/06 12:18:14 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -236,8 +236,11 @@ Parser(void)
   while ((k = Scanner(0)) != 0)	/* end of file */
     {
       if (k != ATOM)
-	Syntax_Error
-	  ("file_name, predicate, directive or ensure_linked expected");
+	{
+	top_error:
+	  Syntax_Error("file_name, predicate, directive "
+		       "or ensure_linked expected");
+	}
 
       if (strcmp(str_val, "file_name") == 0)
 	{
@@ -266,7 +269,7 @@ Parser(void)
       if (strcmp(str_val, "directive") == 0)
 	inside_directive = 1;
       else if (strcmp(str_val, "predicate") != 0)
-	Syntax_Error("file_name, predicate or directive expected");
+	goto top_error;
 
       Read_Token('(');
       if (!inside_directive)
@@ -747,6 +750,9 @@ Peek_Char(int skip_spaces)
   return *p;
 }
 
+
+
+
 /*-------------------------------------------------------------------------*
  * SYNTAX_ERROR                                                            *
  *                                                                         *
